
int INF;
void graphInit(Dict<int, List<int>> E, Dict<int, List<int>> W, int n) {
    for i in range(n) {
        List<int> l;
        List<int> w;
        dictAdd(E, i, l);
        dictAdd(W, i, w);
    }
}

void addEdge(Dict<int, List<int>> E, Dict<int, List<int>> W, int u, int v, int w) {
    List<int> le = dictGetList(E, u);
    List<int> lw = dictGetList(W, u);
    append(le, v);
    append(lw, w);
}

void printGraph(Dict<int, List<int>> E, Dict<int, List<int>> W, int n) {
    for i in range(n) {
        prints("--------------------------");
        print(i);
        List<int> le = dictGetList(E, i);
        List<int> lw = dictGetList(W, i);
        
        if (len(le) > 0) {
            prints("neighbors");
            for v in le {
                print(v);
            }
            prints("weights");
            for w in lw {
                print(w);
            }
        } 
    }
}



bool checkNegativeCycle(Dict<int, List<int>> E, Dict<int, List<int>> W, int src, int n, List<int> dist) {
    /* for every edge */
    for u in range(n) {
        List<int> le = dictGetList(E, u);
        List<int> lw = dictGetList(W, u);
        int m = len(le);
        if (m > 0) {
            for k in range(m) {
                int v = le[k];
                int w = lw[k];

                /* we have u, v, w */
                if (dist[u] != INF && dist[u] + w < dist[v]) {
                    prints("Graph contains negative weight cycle");
                    return false;
                }
            }
        }
    }
    return true;
}

List<int> BellmanFord(Dict<int, List<int>> E, Dict<int, List<int>> W, int src, int n) {
    
    List<int> dist;
    for z in range(n) {
        append(dist, INF);
    } 
    dist[src] = 0;

    /* Loop |V| times */
    for i in range(n - 1) {
        /* for every edge */
        for u in range(n) {
            List<int> le = dictGetList(E, u);
            List<int> lw = dictGetList(W, u);
            int m = len(le);
            if (m > 0) {
                for k in range(m) {
                    int v = le[k];
                    int w = lw[k];

                    /* we have u, v, w */
                    if (dist[u] != INF && dist[u] + w < dist[v]) {
                        dist[v] = dist[u] + w;
                    }
                }
            }
        }
    }
    checkNegativeCycle(E, W, src, n, dist);
    return dist;
}

int main() {
    int n = 5;
    INF = 100000;
    Dict<int, List<int>> E;
    Dict<int, List<int>> W;
    graphInit(E, W, n);
    addEdge(E, W,  0, 1, -1);
    addEdge(E, W,  0, 2, 4);
    addEdge(E, W,  1, 2, 3);
    addEdge(E, W,  1, 3, 2);
    addEdge(E, W,  1, 4, 2);
    addEdge(E, W,  3, 2, 5);
    addEdge(E, W,  3, 1, 1);
    addEdge(E, W,  4, 3, -3);
    printGraph(E, W, n);
    prints("Vertex Distance from Source");
    List<int> dist = BellmanFord(E, W, 0, n);
    for d in dist {
        print(d);
    }

    return 0;
}
